@@ -61,7 +61,7 @@ show_data_sources (DsnConfigPrivate *priv)
 {
 	GList *dsn_list;
 	GList *l;
-	GdaDataModel *model;
+	GdaDataModel *model, *current;
 	GtkTreeViewColumn *col;
 
 	/* create the data model */
@@ -74,10 +74,16 @@ show_data_sources (DsnConfigPrivate *priv)
 	for (l = dsn_list; l != NULL; l = l->next) {
 		GList *value_list;
 		GdaDataSourceInfo *dsn_info = l->data;
+		GValue *tmpval;
 
-		value_list = g_list_append (NULL, gda_value_new_string (dsn_info->name));
-		value_list = g_list_append (value_list, gda_value_new_string (dsn_info->description));
-		value_list = g_list_append (value_list, gda_value_new_boolean (dsn_info->is_global));
+		g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), dsn_info->name);
+		value_list = g_list_append (NULL, tmpval);
+
+		g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), dsn_info->description);
+		value_list = g_list_append (value_list, tmpval);
+
+		g_value_set_boolean (tmpval = gda_value_new (G_TYPE_BOOLEAN), dsn_info->is_global);
+		value_list = g_list_append (value_list, tmpval);
 
 		gda_data_model_append_values (model, (const GList *) value_list, NULL);
 
@@ -88,7 +94,14 @@ show_data_sources (DsnConfigPrivate *priv)
 	gda_config_free_data_source_list (dsn_list);
 
 	/* display the new data model */
-	gnome_db_data_widget_set_gda_model (GNOME_DB_DATA_WIDGET (priv->dsn_list), model);
+	current = gnome_db_data_widget_get_gda_model (GNOME_DB_DATA_WIDGET (priv->dsn_list));
+	if (current) {
+		g_object_set (G_OBJECT (current), "read_only", FALSE, NULL);
+		gda_data_model_import_from_model (current, model, TRUE, NULL, NULL);
+		g_object_set (G_OBJECT (current), "read_only", TRUE, NULL);
+	}
+	else
+		gnome_db_data_widget_set_gda_model (GNOME_DB_DATA_WIDGET (priv->dsn_list), model);
 	g_object_unref (G_OBJECT (model));
 
 	col = gtk_tree_view_get_column (GTK_TREE_VIEW (priv->dsn_list), 1);
@@ -165,6 +178,7 @@ dsn_config_new (void)
 	GtkWidget *box;
 	GtkWidget *button;
 	GtkWidget *label;
+	GtkWidget *sw;
 	gchar *title;
 
 	priv = g_new0 (DsnConfigPrivate, 1);
@@ -196,19 +210,26 @@ dsn_config_new (void)
 	gtk_widget_show (priv->title);
 
 	/* create the data source list */
+	sw = gtk_scrolled_window_new (NULL, NULL);
+	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw), GTK_POLICY_AUTOMATIC,
+					GTK_POLICY_AUTOMATIC);
+	gtk_table_attach (GTK_TABLE (table), sw, 0, 1, 1, 2,
+			  GTK_FILL | GTK_SHRINK | GTK_EXPAND,
+			  GTK_FILL | GTK_SHRINK | GTK_EXPAND,
+			  0, 0);
+	
 	priv->dsn_list = gnome_db_raw_grid_new (NULL);
+	g_object_set_data (G_OBJECT (dsn), "grid", priv->dsn_list);
  	gnome_db_data_widget_column_set_editable (GNOME_DB_DATA_WIDGET (priv->dsn_list), 0, FALSE);
 	g_object_set (priv->dsn_list, "info_cell_visible", FALSE, NULL);
+
+	gtk_container_add (GTK_CONTAINER (sw), priv->dsn_list);
 	
-	gtk_widget_show (priv->dsn_list);
+	gtk_widget_show_all (sw);
 	g_signal_connect (priv->dsn_list, "double_clicked",
 			  G_CALLBACK (list_double_clicked_cb), dsn);
 	g_signal_connect (priv->dsn_list, "populate_popup",
 			  G_CALLBACK (list_popup_cb), dsn);
-	gtk_table_attach (GTK_TABLE (table), priv->dsn_list, 0, 1, 1, 2,
-			  GTK_FILL | GTK_SHRINK | GTK_EXPAND,
-			  GTK_FILL | GTK_SHRINK | GTK_EXPAND,
-			  0, 0);
 	show_data_sources (priv);
 
 	/* add tip */
@@ -258,7 +279,7 @@ dsn_config_edit_properties (GtkWidget *dsn)
 	if (!GDA_IS_DATA_MODEL (model))
 		return;
 
-	str = gda_value_stringify ((GdaValue *) gda_data_model_get_value_at (model, 0, GPOINTER_TO_INT (selection->data)));
+	str = gda_value_stringify ((GValue *) gda_data_model_get_value_at (model, 0, GPOINTER_TO_INT (selection->data)));
 
 	dsn_properties_dialog (GTK_WINDOW (gtk_widget_get_toplevel (dsn)), str);
 
@@ -280,7 +301,7 @@ foreach_remove_cb (GtkTreeModel *tree_model, GtkTreePath *path, GtkTreeIter *ite
 	if (!GDA_IS_DATA_MODEL (model))
 		return;
 
-	dsn = gda_value_stringify ((GdaValue *) gda_data_model_get_value_at (model, 0, row));
+	dsn = gda_value_stringify ((GValue *) gda_data_model_get_value_at (model, 0, row));
 	if (dsn == NULL)
 		return;
 
