@@ -47,14 +47,18 @@ static void gnome_db_basic_form_clean (GnomeDbBasicForm *form);
 
 static void paramlist_destroyed_cb (GdaParameterList *paramlist, GnomeDbBasicForm *form);
 static void paramlist_public_data_changed_cb (GdaParameterList *paramlist, GnomeDbBasicForm *form);
+static void paramlist_param_plugin_changed_cb (GdaParameterList *paramlist, GdaParameter *param, GnomeDbBasicForm *form);
+static void paramlist_param_attr_changed_cb (GdaParameterList *paramlist, GdaParameter *param, GnomeDbBasicForm *form);
 
 static void entry_contents_modified (GnomeDbDataEntry *entry, GnomeDbBasicForm *form);
+static void entry_contents_activated (GnomeDbDataEntry *entry, GnomeDbBasicForm *form);
 static void parameter_changed_cb (GdaParameter *param, GnomeDbDataEntry *entry);
 
 static void mark_not_null_entry_labels (GnomeDbBasicForm *form, gboolean show_mark);
 enum
 {
 	PARAM_CHANGED,
+	ACTIVATED,
 	LAST_SIGNAL
 };
 
@@ -90,7 +94,7 @@ struct _GnomeDbBasicFormPriv
 };
 
 
-static gint gnome_db_basic_form_signals[LAST_SIGNAL] = { 0 };
+static gint gnome_db_basic_form_signals[LAST_SIGNAL] = { 0, 0 };
 
 /* get a pointer to the parents to be able to call their destructor */
 static GObjectClass *parent_class = NULL;
@@ -135,8 +139,16 @@ gnome_db_basic_form_class_init (GnomeDbBasicFormClass * class)
 			      NULL, NULL,
 			      marshal_VOID__OBJECT_BOOLEAN, G_TYPE_NONE, 2,
 			      G_TYPE_OBJECT, G_TYPE_BOOLEAN);
+	gnome_db_basic_form_signals[ACTIVATED] =
+		g_signal_new ("activated",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_FIRST,
+			      G_STRUCT_OFFSET (GnomeDbBasicFormClass, activated),
+			      NULL, NULL,
+			      marshal_VOID__VOID, G_TYPE_NONE, 0);
 
 	class->param_changed = NULL;
+	class->activated = NULL;
 	object_class->dispose = gnome_db_basic_form_dispose;
 
 	/* Properties */
@@ -282,6 +294,11 @@ paramlist_destroyed_cb (GdaParameterList *paramlist, GnomeDbBasicForm *form)
 					      G_CALLBACK (paramlist_destroyed_cb), form);
 	g_signal_handlers_disconnect_by_func (paramlist,
 					      G_CALLBACK (paramlist_public_data_changed_cb), form);
+	g_signal_handlers_disconnect_by_func (paramlist,
+					      G_CALLBACK (paramlist_param_plugin_changed_cb), form);
+	g_signal_handlers_disconnect_by_func (paramlist,
+					      G_CALLBACK (paramlist_param_attr_changed_cb), form);
+
 	g_object_unref (form->priv->paramlist);
 	form->priv->paramlist = NULL;
 
@@ -301,13 +318,51 @@ paramlist_public_data_changed_cb (GdaParameterList *paramlist, GnomeDbBasicForm
 	gnome_db_basic_form_fill (form);
 }
 
+static void
+paramlist_param_plugin_changed_cb (GdaParameterList *paramlist, GdaParameter *param, GnomeDbBasicForm *form)
+{
+	/* TODO: be more specific and change only the entry corresponding to @param */
+	paramlist_public_data_changed_cb (paramlist, form);
+}
+
+static void
+paramlist_param_attr_changed_cb (GdaParameterList *paramlist, GdaParameter *param, GnomeDbBasicForm *form)
+{
+	GtkWidget *entry;
+
+	entry = gnome_db_basic_form_get_entry_widget (form, param);
+	if (entry) {
+		gboolean toset;
+		guint attrs = 0;
+		guint mask = 0;
+
+		toset = gda_parameter_get_exists_default_value (param);
+		attrs |= toset ? GDA_VALUE_ATTR_CAN_BE_DEFAULT : 0;
+		mask |= GDA_VALUE_ATTR_CAN_BE_DEFAULT;
+
+		toset = gda_parameter_get_not_null (param);
+		attrs |= toset ? 0 : GDA_VALUE_ATTR_CAN_BE_NULL;
+		mask |= GDA_VALUE_ATTR_CAN_BE_NULL;
+
+		g_object_get (G_OBJECT (param), "use-default-value", &toset, NULL);
+		attrs |= toset ? GDA_VALUE_ATTR_IS_DEFAULT : 0;
+		mask |= GDA_VALUE_ATTR_IS_DEFAULT;
+
+		g_signal_handlers_block_by_func (G_OBJECT (entry),
+						 G_CALLBACK (entry_contents_modified), form);
+		gnome_db_data_entry_set_attributes (GNOME_DB_DATA_ENTRY (entry), attrs, mask);
+		g_signal_handlers_unblock_by_func (G_OBJECT (entry),
+						   G_CALLBACK (entry_contents_modified), form);
+	}
+}
+
 static void
 gnome_db_basic_form_dispose (GObject *object)
 {
 	GnomeDbBasicForm *form;
 
 	g_return_if_fail (object != NULL);
-	g_return_if_fail (IS_GNOME_DB_BASIC_FORM (object));
+	g_return_if_fail (GNOME_DB_IS_BASIC_FORM (object));
 	form = GNOME_DB_BASIC_FORM (object);
 
 	if (form->priv) {
@@ -401,6 +456,10 @@ gnome_db_basic_form_set_property (GObject *object,
 							    G_CALLBACK (paramlist_destroyed_cb), form);
 				g_signal_connect (form->priv->paramlist, "public_data_changed",
 						  G_CALLBACK (paramlist_public_data_changed_cb), form);
+				g_signal_connect (form->priv->paramlist, "param_plugin_changed",
+						  G_CALLBACK (paramlist_param_plugin_changed_cb), form);
+				g_signal_connect (form->priv->paramlist, "param_attr_changed",
+						  G_CALLBACK (paramlist_param_attr_changed_cb), form);
 				
 				gnome_db_basic_form_fill (form);
 			}
@@ -628,6 +687,8 @@ gnome_db_basic_form_fill (GnomeDbBasicForm *form)
 		/* connect the entry's changes */
 		g_signal_connect (G_OBJECT (entry), "contents_modified",
 				  G_CALLBACK (entry_contents_modified), form);
+		g_signal_connect (G_OBJECT (entry), "contents_activated",
+				  G_CALLBACK (entry_contents_activated), form);
 		list = g_slist_next (list);
 	}
 
@@ -705,21 +766,22 @@ gnome_db_basic_form_fill (GnomeDbBasicForm *form)
 			/* label for the entry */
 			param = g_object_get_data (G_OBJECT (list->data), "param");
 			if (param) {
-				const gchar *str;
-				gchar *str2;
+				const gchar *title;
+				gchar *str;
 				GtkWidget *evbox;
-				
-				str = gda_object_get_name (GDA_OBJECT (param));
-				if (!str)
-					str = _("Value");
-				str2 = g_strdup_printf ("%s:", str);
+
+				title = gda_object_get_name (GDA_OBJECT (param));
+				if (!title)
+					title = _("Value");
 				evbox = gtk_event_box_new ();
-				label = gtk_label_new (str2);
+				str = g_strdup_printf ("%s:", title);
+				label = gtk_label_new (str);
+				g_free (str);
+				g_object_set_data_full (G_OBJECT (label), "_gda_title", g_strdup (title), g_free);
 				if (gda_parameter_get_not_null (param)) 
 					form->priv->not_null_labels = g_slist_prepend (form->priv->not_null_labels, label);
 				
 				gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
-				g_free (str2);
 				gtk_container_add (GTK_CONTAINER (evbox), label);
 				
 				gtk_table_attach (GTK_TABLE (table), evbox, 0, 1, i, i+1,
@@ -728,14 +790,14 @@ gnome_db_basic_form_fill (GnomeDbBasicForm *form)
 				gtk_widget_show (label);
 				entry_label = evbox;
 				
-				str = gda_object_get_description (GDA_OBJECT (param));
-				if (str && *str)
-					gtk_tooltips_set_tip (form->priv->tooltips, evbox, str, NULL);
+				title = gda_object_get_description (GDA_OBJECT (param));
+				if (title && *title)
+					gtk_tooltips_set_tip (form->priv->tooltips, evbox, title, NULL);
 			}
 			else {
 				/* FIXME: find a better label and tooltip and improve data entry attributes */
-				const gchar *str, *title = NULL;
-				gchar *str2;
+				const gchar *title = NULL;
+				gchar *str;
 				GtkWidget *evbox;
 				gboolean nullok = TRUE;
 				GSList *params;
@@ -755,13 +817,14 @@ gnome_db_basic_form_fill (GnomeDbBasicForm *form)
 					title = gda_object_get_name (GDA_OBJECT (group->nodes_source->data_model));
 				if (!title)
 					title = _("Value");
-				str2 = g_strdup_printf ("%s:", title);
 				evbox = gtk_event_box_new ();
-				label = gtk_label_new (str2);
+				str = g_strdup_printf ("%s:", title);
+				label = gtk_label_new (str);
+				g_free (str);
+				g_object_set_data_full (G_OBJECT (label), "_gda_title", g_strdup (title), g_free);
 				if (!nullok) 
 					form->priv->not_null_labels = g_slist_prepend (form->priv->not_null_labels, label);
 				gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
-				g_free (str2);
 				gtk_container_add (GTK_CONTAINER (evbox), label);
 				
 				gtk_table_attach (GTK_TABLE (table), evbox, 0, 1, i, i+1,
@@ -770,9 +833,9 @@ gnome_db_basic_form_fill (GnomeDbBasicForm *form)
 				gtk_widget_show (label);
 				entry_label = evbox;
 				
-				str = gda_object_get_description (GDA_OBJECT (group->nodes_source->data_model));
-				if (str && *str)
-					gtk_tooltips_set_tip (form->priv->tooltips, evbox, str, NULL);
+				title = gda_object_get_description (GDA_OBJECT (group->nodes_source->data_model));
+				if (title && *title)
+					gtk_tooltips_set_tip (form->priv->tooltips, evbox, title, NULL);
 			}
 
 			/* add the entry itself to the table */
@@ -839,26 +902,35 @@ label_destroyed_cb (GtkWidget *label, GnomeDbBasicForm *form)
 static void
 mark_not_null_entry_labels (GnomeDbBasicForm *form, gboolean show_mark)
 {
-	PangoAttrList *attrs = NULL;
-	PangoAttribute *att;
 	GSList *list;
 
-	if (show_mark) {
-		attrs = pango_attr_list_new ();
-		att = pango_attr_weight_new (PANGO_WEIGHT_BOLD);
-		att->start_index = 0;
-		att->end_index = G_MAXUINT;
-		pango_attr_list_insert (attrs, att);
-	}
+	for (list = form->priv->not_null_labels; list; list = list->next) {
+		const gchar *title;
+		gchar *str;
 
-	list = form->priv->not_null_labels;
-	while (list) {
-		gtk_label_set_attributes (GTK_LABEL (list->data), attrs);
-		list = g_slist_next (list);
+		title = g_object_get_data (G_OBJECT (list->data), "_gda_title");
+		if (show_mark) {
+			str = g_strdup_printf ("%s <span foreground='red' weight='bold'>*</span>:", title);
+			gtk_label_set_markup (GTK_LABEL (list->data), str);
+		}
+		else {
+			str = g_strdup_printf ("%s:", title);
+			gtk_label_set_text (GTK_LABEL (list->data), str);			
+		}
+		g_free (str);
 	}
+}
 
-	if (show_mark)
-		pango_attr_list_unref (attrs);
+static void
+entry_contents_activated (GnomeDbDataEntry *entry, GnomeDbBasicForm *form)
+{
+#ifdef debug_signal
+	g_print (">> 'ACTIVATED' from %s\n", __FUNCTION__);
+#endif
+	g_signal_emit (G_OBJECT (form), gnome_db_basic_form_signals[ACTIVATED], 0);
+#ifdef debug_signal
+	g_print ("<< 'ACTIVATED' from %s\n", __FUNCTION__);
+#endif
 }
 
 static void
@@ -959,6 +1031,7 @@ entry_contents_modified (GnomeDbDataEntry *entry, GnomeDbBasicForm *form)
 				}
 				g_slist_free (all_new_values);
 			}
+			g_object_unref (proxy);
 		}
 #endif
 	}
@@ -989,9 +1062,12 @@ parameter_changed_cb (GdaParameter *param, GnomeDbDataEntry *entry)
 				g_object_get (G_OBJECT (entry), "set_default_if_invalid", &default_if_invalid, NULL);
 
 		/* updating the corresponding entry */
-		if (! default_if_invalid)
+		if (! default_if_invalid) {
 			g_signal_handlers_block_by_func (G_OBJECT (entry),
 							 G_CALLBACK (entry_contents_modified), form);
+			g_signal_handlers_block_by_func (G_OBJECT (entry),
+							 G_CALLBACK (entry_contents_activated), form);
+		}
 		if (group) {
 			GSList *values = NULL;
 			GSList *list = group->nodes;
@@ -1018,9 +1094,12 @@ parameter_changed_cb (GdaParameter *param, GnomeDbDataEntry *entry)
 		else
 			gnome_db_data_entry_set_value (entry, value);
 
-		if (! default_if_invalid)
+		if (! default_if_invalid) {
 			g_signal_handlers_unblock_by_func (G_OBJECT (entry),
 							   G_CALLBACK (entry_contents_modified), form);
+			g_signal_handlers_unblock_by_func (G_OBJECT (entry),
+							   G_CALLBACK (entry_contents_activated), form);
+		}
 
 #ifdef debug_signal
 		g_print (">> 'PARAM_CHANGED' from %s\n", __FUNCTION__);
@@ -1044,7 +1123,7 @@ parameter_changed_cb (GdaParameter *param, GnomeDbDataEntry *entry)
 GdaParameterList *
 gnome_db_basic_form_get_paramlist (GnomeDbBasicForm *form)
 {
-	g_return_val_if_fail (form && IS_GNOME_DB_BASIC_FORM (form), NULL);
+	g_return_val_if_fail (form && GNOME_DB_IS_BASIC_FORM (form), NULL);
 	g_return_val_if_fail (form->priv, NULL);
 
 	return form->priv->paramlist;
@@ -1065,7 +1144,7 @@ gnome_db_basic_form_set_current_as_orig (GnomeDbBasicForm *form)
 	GSList *list;
 	GdaParameter *param;
 
-	g_return_if_fail (form && IS_GNOME_DB_BASIC_FORM (form));
+	g_return_if_fail (form && GNOME_DB_IS_BASIC_FORM (form));
 	g_return_if_fail (form->priv);
 
 	list = form->priv->entries;
@@ -1118,7 +1197,7 @@ gnome_db_basic_form_set_current_as_orig (GnomeDbBasicForm *form)
 gboolean
 gnome_db_basic_form_is_valid (GnomeDbBasicForm *form)
 {
-	g_return_val_if_fail (form && IS_GNOME_DB_BASIC_FORM (form), FALSE);
+	g_return_val_if_fail (form && GNOME_DB_IS_BASIC_FORM (form), FALSE);
 	g_return_val_if_fail (form->priv, FALSE);
 
 	return gda_parameter_list_is_valid (form->priv->paramlist);
@@ -1136,7 +1215,7 @@ gnome_db_basic_form_is_valid (GnomeDbBasicForm *form)
 GdaParameterList *
 gnome_db_basic_form_get_data_set (GnomeDbBasicForm *form)
 {
-	g_return_val_if_fail (form && IS_GNOME_DB_BASIC_FORM (form), NULL);
+	g_return_val_if_fail (form && GNOME_DB_IS_BASIC_FORM (form), NULL);
 	g_return_val_if_fail (form->priv, NULL);
 
 	return form->priv->paramlist;
@@ -1156,7 +1235,7 @@ gnome_db_basic_form_has_been_changed (GnomeDbBasicForm *form)
 	gboolean changed = FALSE;
 	GSList *list;
 
-	g_return_val_if_fail (form && IS_GNOME_DB_BASIC_FORM (form), FALSE);
+	g_return_val_if_fail (form && GNOME_DB_IS_BASIC_FORM (form), FALSE);
 	g_return_val_if_fail (form->priv, FALSE);
 	
 	list = form->priv->entries;
@@ -1183,7 +1262,7 @@ gnome_db_basic_form_show_entries_actions (GnomeDbBasicForm *form, gboolean show_
 	GSList *entries;
 	guint show;
 	
-	g_return_if_fail (form && IS_GNOME_DB_BASIC_FORM (form));
+	g_return_if_fail (form && GNOME_DB_IS_BASIC_FORM (form));
 	g_return_if_fail (form->priv);
 
 	show = show_actions ? GDA_VALUE_ATTR_ACTIONS_SHOWN : 0;
@@ -1211,7 +1290,7 @@ gnome_db_basic_form_reset (GnomeDbBasicForm *form)
 {
 	GSList *list;
 
-	g_return_if_fail (form && IS_GNOME_DB_BASIC_FORM (form));
+	g_return_if_fail (form && GNOME_DB_IS_BASIC_FORM (form));
 	g_return_if_fail (form->priv);
 	
 	list = form->priv->entries;
@@ -1252,7 +1331,7 @@ gnome_db_basic_form_entry_show (GnomeDbBasicForm *form, GdaParameter *param, gbo
 {
 	GtkWidget *entry;
 
-	g_return_if_fail (form && IS_GNOME_DB_BASIC_FORM (form));
+	g_return_if_fail (form && GNOME_DB_IS_BASIC_FORM (form));
 	g_return_if_fail (form->priv);
 
 	entry = gnome_db_basic_form_get_entry_widget (form, param);
@@ -1307,6 +1386,27 @@ gnome_db_basic_form_entry_show (GnomeDbBasicForm *form, GdaParameter *param, gbo
 	}
 }
 
+/**
+ * gnome_db_basic_form_entry_grab_focus
+ * @form: a #GnomeDbBasicForm widget
+ * @param: a #GdaParameter object
+ * 
+ * Makes the data entry corresponding to @param grab the focus for the window it's in
+ */
+void
+gnome_db_basic_form_entry_grab_focus (GnomeDbBasicForm *form, GdaParameter *param)
+{
+	GtkWidget *entry;
+
+	g_return_if_fail (form && GNOME_DB_IS_BASIC_FORM (form));
+	g_return_if_fail (form->priv);
+
+	entry = gnome_db_basic_form_get_entry_widget (form, param);
+
+	if (entry)
+		gnome_db_data_entry_grab_focus (GNOME_DB_DATA_ENTRY (entry));
+}
+
 /**
  * gnome_db_basic_form_entry_set_editable
  * @form: a #GnomeDbBasicForm widget
@@ -1321,7 +1421,7 @@ gnome_db_basic_form_entry_set_editable (GnomeDbBasicForm *form, GdaParameter *pa
 {
 	GtkWidget *entry;
 
-	g_return_if_fail (form && IS_GNOME_DB_BASIC_FORM (form));
+	g_return_if_fail (form && GNOME_DB_IS_BASIC_FORM (form));
 	g_return_if_fail (form->priv);
 
 	entry = gnome_db_basic_form_get_entry_widget (form, param);
@@ -1352,7 +1452,7 @@ gnome_db_basic_form_set_entries_auto_default (GnomeDbBasicForm *form, gboolean a
 {
 	GSList *entries;
 
-	g_return_if_fail (form && IS_GNOME_DB_BASIC_FORM (form));
+	g_return_if_fail (form && GNOME_DB_IS_BASIC_FORM (form));
 	g_return_if_fail (form->priv);
 	
 	form->priv->entries_auto_default = auto_default;
@@ -1376,7 +1476,7 @@ gnome_db_basic_form_set_entries_default (GnomeDbBasicForm *form)
 	GSList *entries;
 	guint attrs;
 
-	g_return_if_fail (form && IS_GNOME_DB_BASIC_FORM (form));
+	g_return_if_fail (form && GNOME_DB_IS_BASIC_FORM (form));
 	g_return_if_fail (form->priv);
 
 	entries = form->priv->entries;
@@ -1384,7 +1484,7 @@ gnome_db_basic_form_set_entries_default (GnomeDbBasicForm *form)
 		attrs = gnome_db_data_entry_get_attributes (GNOME_DB_DATA_ENTRY (entries->data));
 		if (attrs & GDA_VALUE_ATTR_CAN_BE_DEFAULT)
 			gnome_db_data_entry_set_attributes (GNOME_DB_DATA_ENTRY (entries->data), 
-						      GDA_VALUE_ATTR_IS_DEFAULT, GDA_VALUE_ATTR_IS_DEFAULT);
+							    GDA_VALUE_ATTR_IS_DEFAULT, GDA_VALUE_ATTR_IS_DEFAULT);
 		entries = g_slist_next (entries);
 	}
 }
@@ -1406,7 +1506,7 @@ gnome_db_basic_form_get_entry_widget (GnomeDbBasicForm *form, GdaParameter *para
 	GSList *entries;
 	GtkWidget *entry = NULL;
 
-	g_return_val_if_fail (IS_GNOME_DB_BASIC_FORM (form), NULL);
+	g_return_val_if_fail (GNOME_DB_IS_BASIC_FORM (form), NULL);
 	g_return_val_if_fail (form->priv, NULL);
 
 	entries = form->priv->entries;
@@ -1451,7 +1551,7 @@ gnome_db_basic_form_get_label_widget (GnomeDbBasicForm *form, GdaParameter *para
 {
 	GtkWidget *entry;
 
-	g_return_val_if_fail (IS_GNOME_DB_BASIC_FORM (form), NULL);
+	g_return_val_if_fail (GNOME_DB_IS_BASIC_FORM (form), NULL);
 	g_return_val_if_fail (form->priv, NULL);
 
 	entry = gnome_db_basic_form_get_entry_widget (form, param);
