@@ -138,7 +138,7 @@ gnome_db_dbms_update_viewer_dispose (GObject *object)
 	GnomeDbmsUpdateViewer *mgv;
 
 	g_return_if_fail (object != NULL);
-	g_return_if_fail (IS_GNOME_DB_DBMS_UPDATE_VIEWER (object));
+	g_return_if_fail (GNOME_DB_IS_DBMS_UPDATE_VIEWER (object));
 	mgv = GNOME_DB_DBMS_UPDATE_VIEWER (object);
 
 	if (mgv->priv) {
@@ -222,6 +222,7 @@ gnome_db_dbms_update_viewer_do_init (GnomeDbmsUpdateViewer *mgv)
 	}
 	append_refresh_element_table (mgv, i++, NULL, NULL);
 	append_refresh_element_table (mgv, i++, "TABLES", _("Tables analysis"));
+	append_refresh_element_table (mgv, i++, "VIEWS", _("Views analysis"));
 	/*append_refresh_element_table (mgv, i++, "CONSTRAINTS", _("Database constraints analysis"));*/
 
 	/* progress bar */
@@ -349,6 +350,8 @@ update_progress_cb (GdaConnection *cnc, gchar *key, guint now, guint total, Gnom
 			str = g_strdup_printf ("%sN", key);
 			wid = g_object_get_data (G_OBJECT (table), str);
 			g_free (str);
+			if (!wid)
+				g_error (_("Missing description regarding update progress of %s"), key);
 			gtk_widget_show (wid);
 			
 			/* description */
@@ -421,7 +424,7 @@ update_progress_cb (GdaConnection *cnc, gchar *key, guint now, guint total, Gnom
 void
 gnome_db_dbms_update_viewer_reset (GnomeDbmsUpdateViewer *mgv)
 {
-	g_return_if_fail (mgv && IS_GNOME_DB_DBMS_UPDATE_VIEWER (mgv));
+	g_return_if_fail (mgv && GNOME_DB_IS_DBMS_UPDATE_VIEWER (mgv));
 	g_return_if_fail (mgv->priv);
 	
 	gnome_db_dbms_update_viewer_do_reset (mgv, TRUE);
