@@ -29,7 +29,7 @@ G_BEGIN_DECLS
 #define GNOME_DB_TYPE_DBMS_UPDATE_VIEWER          (gnome_db_dbms_update_viewer_get_type())
 #define GNOME_DB_DBMS_UPDATE_VIEWER(obj)          G_TYPE_CHECK_INSTANCE_CAST (obj, gnome_db_dbms_update_viewer_get_type(), GnomeDbmsUpdateViewer)
 #define GNOME_DB_DBMS_UPDATE_VIEWER_CLASS(klass)  G_TYPE_CHECK_CLASS_CAST (klass, gnome_db_dbms_update_viewer_get_type (), GnomeDbmsUpdateViewerClass)
-#define IS_GNOME_DB_DBMS_UPDATE_VIEWER(obj)       G_TYPE_CHECK_INSTANCE_TYPE (obj, gnome_db_dbms_update_viewer_get_type ())
+#define GNOME_DB_IS_DBMS_UPDATE_VIEWER(obj)       G_TYPE_CHECK_INSTANCE_TYPE (obj, gnome_db_dbms_update_viewer_get_type ())
 
 
 typedef struct _GnomeDbmsUpdateViewer      GnomeDbmsUpdateViewer;
