@@ -1,5 +1,5 @@
 /* GNOME DB library
- * Copyright (C) 1999 - 2005 The GNOME Foundation
+ * Copyright (C) 1999 - 2006 The GNOME Foundation
  *
  * AUTHORS:
  *      Rodrigo Moya <rodrigo@gnome-db.org>
@@ -24,22 +24,24 @@
 #include <string.h>
 #include <libgda/gda-client.h>
 #include <libgda/gda-config.h>
+#include <libgda/gda-parameter-list.h>
+#include <libgda/gda-parameter.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <libgnomeui/gnome-druid-page-edge.h>
 #include <libgnomeui/gnome-druid-page-standard.h>
 #include <libgnomedb/gnome-db-dsn-config-druid.h>
+#include <libgnomedb/gnome-db-dsn-spec.h>
 #include <libgnomedb/gnome-db-provider-selector.h>
 #include <libgnomedb/gnome-db-util.h>
-#include <libgnomedb/gnome-db-data-set.h>
-#include <libgnomedb/gnome-db-parameter.h>
-#include <libgnomedb/gnome-db-basic-form.h>
+#include <libgnomedb/gnome-db-server-operation.h>
 #include <glib/gi18n-lib.h>
 
 #define PARENT_TYPE GNOME_TYPE_DRUID
 
 struct _GnomeDbDsnConfigDruidPrivate {
-	GdaClient *client;
+	GdaClient         *client;
 	GdaDataSourceInfo *dsn_info;
+	GdaServerOperation *create_db_op;
 
 	GdkPixbuf *logo;
 
@@ -53,7 +55,6 @@ struct _GnomeDbDsnConfigDruidPrivate {
 	GtkWidget *general_username;
 	GtkWidget *general_password;
 	GtkWidget *general_is_global;
-	gchar     *create_db_spec;
 
 	GtkWidget *choose_page;
 	GtkWidget *choose_toggle;
@@ -66,10 +67,7 @@ struct _GnomeDbDsnConfigDruidPrivate {
 	GtkWidget *provider_container;
 	GtkWidget *provider_detail;
 
-	GHashTable *provider_params;
-	GHashTable *remembered_values;
-
-	GtkWidget *end_page;
+	GtkWidget  *end_page;
 };
 
 static void gnome_db_dsn_config_druid_class_init (GnomeDbDsnConfigDruidClass *klass);
@@ -85,84 +83,24 @@ enum {
 static guint config_druid_signals[LAST_SIGNAL] = { 0, };
 static GObjectClass *parent_class = NULL;
 
-/*
- * Private functions
- */
-
-static void
-add_to_string (gpointer key, gpointer value, gpointer user_data)
-{
-	const gchar *contents;
-	GString **str = (GString **) user_data;
-
-
-	if (GTK_IS_ENTRY (value)) {
-		contents = gtk_entry_get_text (GTK_ENTRY (value));
-		if (!contents || !contents[0])
-			return;
-
-		if (*str)
-			*str = g_string_append (*str, ";");
-		else
-			*str = g_string_new ("");
-
-		*str = g_string_append (*str, (const gchar *) key);
-		*str = g_string_append (*str, "=");
-		*str = g_string_append (*str, contents);
-	}
-}
-
-static gchar *
-params_to_string (GnomeDbDsnConfigDruid *druid)
-{
-	GString *str = NULL;
-	gchar *retval;
-
-	g_return_val_if_fail (GNOME_DB_IS_DSN_CONFIG_DRUID (druid), NULL);
-
-	g_hash_table_foreach (druid->priv->provider_params, (GHFunc) add_to_string, &str);
-	if (str) {
-		retval = str->str;
-		g_string_free (str, FALSE);
-	}
-	else
-		retval = g_strdup ("");
-
-	return retval;
-}
-
-static gboolean
-remove_hash_func (gpointer key, gpointer value, gpointer user_data)
-{
-	g_free (key);
-	return TRUE;
-}
-
-static gboolean
-remove_remembered_hash (gpointer key, gpointer value, gpointer user_data)
-{
-	g_free (key);
-	g_free (value);
-	return TRUE;
-}
-
 /*
  * Callbacks
  */
 
-static gchar *
+static GdaServerOperation *
 get_specs_database_creation (GnomeDbDsnConfigDruid *druid)
 {
-	if (! druid->priv->create_db_spec) {
+	if (! druid->priv->create_db_op) {
 		if (!druid->priv->client)
 			druid->priv->client = gda_client_new ();
 		
-		druid->priv->create_db_spec = gda_client_get_specs_to_create_database (druid->priv->client, 
-								gnome_db_provider_selector_get_selected_provider (
-								GNOME_DB_PROVIDER_SELECTOR (druid->priv->general_provider)));
+		druid->priv->create_db_op = 
+			gda_client_prepare_create_database (druid->priv->client, NULL,
+					  gnome_db_provider_selector_get_selected_provider (
+					  GNOME_DB_PROVIDER_SELECTOR (druid->priv->general_provider)));
 	}
 
-	return druid->priv->create_db_spec;
+	return druid->priv->create_db_op;
 }
 
 static void
@@ -190,91 +128,62 @@ druid_finished_cb (GnomeDruidPage *druid_page, GtkWidget *druid_widget, GnomeDbD
 
 	/* New database creation first */
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (druid->priv->choose_toggle))) {
-		/* make a list of GdaParameter parameters */
-		GnomeDbDataSet *dset;
-
-		dset = gnome_db_basic_form_get_data_set (GNOME_DB_BASIC_FORM (druid->priv->newdb_params));
-		if (!dset) 
-			return;
-
-		if (!gnome_db_data_set_is_valid (dset)) {
-			gnome_db_show_error (_("Missing mandatory informmation, to create database"));
+		if (!gda_server_operation_is_valid (druid->priv->create_db_op, NULL, NULL)) {
+			gnome_db_show_error (NULL, _("Missing mandatory informmation, to create database"));
 			gnome_druid_set_page (GNOME_DRUID (druid), GNOME_DRUID_PAGE (druid->priv->newdb_page));
 			return;
 		}
 		else {
-			GdaParameterList *plist = NULL;
-			GSList *params;
-			GdaParameter *gda_param;
-			GnomeDbParameter *param;
-			gchar *str;
-			GError *error = NULL;
-			GList *dsn_params;
-
 			GdaProviderInfo *prov_info;
+			GSList *dsn_params;
+			GError *error = NULL;
 
-			params = dset->parameters;
-			while (params) {
-				param = GNOME_DB_PARAMETER (params->data);
-				gda_param = gda_parameter_new_from_value ("", gnome_db_parameter_get_value (param));
-				g_object_get (G_OBJECT (param), "string_id", &str, NULL);
-				if (str) {
-					gda_parameter_set_name (gda_param, str);
-					g_free (str);
-				}
-				else
-					gda_parameter_set_name (gda_param, gnome_db_base_get_name (GNOME_DB_BASE (param)));	
-				if (!plist)
-					plist = gda_parameter_list_new ();
-				gda_parameter_list_add_parameter (plist, gda_param);
-
-				params = g_slist_next (params);
-			}
-
-			allok = gda_client_create_database (druid->priv->client, 
-							    gnome_db_provider_selector_get_selected_provider (
-								    GNOME_DB_PROVIDER_SELECTOR (druid->priv->general_provider)),
-							    plist, &error);
+			allok = gda_client_perform_create_database (druid->priv->client, 
+								    druid->priv->create_db_op, &error);
 			if (!allok) {
+				gchar *str;
 				str = g_strdup_printf (_("Error creating database: %s"), 
 						       error && error->message ? error->message : _("Unknown error"));
-				gnome_db_show_error (str);
+				gnome_db_show_error (NULL, str);
 				g_free (str);
-
+				
 				gnome_druid_set_page (GNOME_DRUID (druid), GNOME_DRUID_PAGE (druid->priv->newdb_page));
-				gda_parameter_list_free (plist);
 				return;
 			}
-
+			
 			/* make the connection string for the data source */
-			prov_info = gda_config_get_provider_by_name (gnome_db_provider_selector_get_selected_provider (
-									     GNOME_DB_PROVIDER_SELECTOR (druid->priv->general_provider)));
+			prov_info = gda_config_get_provider_by_name (
+                                             gnome_db_provider_selector_get_selected_provider (
+					     GNOME_DB_PROVIDER_SELECTOR (druid->priv->general_provider)));
 			g_return_if_fail (prov_info);
-			dsn_params = prov_info->gda_params;
+			dsn_params = prov_info->gda_params->parameters;
 			while (dsn_params) {
-				GdaParameter *param;
+				GdaParameter *param = GDA_PARAMETER (dsn_params->data);
+				const GValue *value;
+				
+				value = gda_server_operation_get_value_at (druid->priv->create_db_op,
+									   "/DB_DEF_P/%s",
+									   gda_object_get_id (GDA_OBJECT (param)));
+				if (!value)
+					value = gda_server_operation_get_value_at (druid->priv->create_db_op,
+									   "/SERVER_CNX_P/%s",
+									   gda_object_get_id (GDA_OBJECT (param)));
+				
+				if (value && !gda_value_is_null ((GValue *) value)) {
+					gchar *str;
 
-				param = gda_parameter_list_find (plist, ((GdaProviderParameterInfo*)(dsn_params->data))->name);
-				if (param) {
-					const GdaValue *value;
-
-					value = gda_parameter_get_value (param);
-					if (value && !gda_value_is_null (value)) {
-						if (dsn_params == prov_info->gda_params)
-							cnc_string = g_string_new ("");
-						else
-							g_string_append (cnc_string, "; ");
-						str = gda_value_stringify (value);
-						g_string_append_printf (cnc_string, "%s=%s", 
-									((GdaProviderParameterInfo*)(dsn_params->data))->name, str);
-						g_free (str);
-					}
+					if (dsn_params == prov_info->gda_params->parameters)
+						cnc_string = g_string_new ("");
+					else
+						g_string_append (cnc_string, ";");
+					str = gda_value_stringify ((GValue *) value);
+					g_string_append_printf (cnc_string, "%s=%s",
+								gda_object_get_id (GDA_OBJECT (param)), str);
+					g_free (str);
 				}
-
-				dsn_params = g_list_next (dsn_params);
+				
+				dsn_params = g_slist_next (dsn_params);
 			}
-
-			gda_parameter_list_free (plist);
 		}
 	}
 
@@ -282,15 +191,16 @@ druid_finished_cb (GnomeDruidPage *druid_page, GtkWidget *druid_widget, GnomeDbD
 	if (allok) {
 		druid->priv->dsn_info = g_new0 (GdaDataSourceInfo, 1);
 		druid->priv->dsn_info->name = g_strdup (gtk_entry_get_text (GTK_ENTRY (druid->priv->general_name)));
-		druid->priv->dsn_info->provider = g_strdup (gnome_db_provider_selector_get_selected_provider (
-								    GNOME_DB_PROVIDER_SELECTOR (druid->priv->general_provider)));
+		druid->priv->dsn_info->provider = g_strdup (
+					 gnome_db_provider_selector_get_selected_provider (
+					 GNOME_DB_PROVIDER_SELECTOR (druid->priv->general_provider)));
 		if (cnc_string) {
-			g_printf ("CNC_STRING:%s\n", cnc_string->str);
 			druid->priv->dsn_info->cnc_string = cnc_string->str;
 			g_string_free (cnc_string, FALSE);
 		}
 		else
-			druid->priv->dsn_info->cnc_string = params_to_string (druid);
+			druid->priv->dsn_info->cnc_string = gnome_db_dsn_spec_get_specs 
+				(GNOME_DB_DSN_SPEC (druid->priv->provider_detail));
 		druid->priv->dsn_info->description =
 			g_strdup (gtk_entry_get_text (GTK_ENTRY (druid->priv->general_description)));
 		druid->priv->dsn_info->username =
@@ -320,9 +230,9 @@ provider_changed_cb (GtkWidget *combo, GnomeDbDsnConfigDruid *druid)
 	}
 
 	/* is the database creation supported by the chosen provider? */
-	if (druid->priv->create_db_spec) {
-		g_free (druid->priv->create_db_spec);
-		druid->priv->create_db_spec = NULL;
+	if (druid->priv->create_db_op) {
+		g_object_unref (druid->priv->create_db_op);
+		druid->priv->create_db_op = NULL;
 	}
 }
 
@@ -332,9 +242,8 @@ general_next_pressed_cb (GnomeDruidPage *druid_page,
 			 GnomeDbDsnConfigDruid *druid)
 {
 	GdaDataSourceInfo *dsn_info;
+	GdaServerOperation *op;
 	GdaProviderInfo *prov_info;
-	GList *l;
-	gint i;
 	const gchar *name;
 	const gchar *provider;
 	gchar *str;
@@ -344,43 +253,23 @@ general_next_pressed_cb (GnomeDruidPage *druid_page,
 	/* check required fields have values */
 	name = gtk_entry_get_text (GTK_ENTRY (druid->priv->general_name));
 	if (!name || strlen (name) < 1) {
-		gnome_db_show_error (_("You must enter a name for the new data source"));
+		gnome_db_show_error (NULL, _("You must enter a name for the new data source"));
 		gtk_widget_grab_focus (druid->priv->general_name);
 		return TRUE;
 	}
 
-	for (i = 0; i < strlen (name); i++) {
-		if (!g_ascii_isalnum (name[i])) {
-			gnome_db_show_error (_("Data source names can only contain alphanumeric characters"));
-			gtk_widget_grab_focus (druid->priv->general_name);
-			return TRUE;
-		}
-	}
-
 	dsn_info = gda_config_find_data_source (name);
 	if (dsn_info != NULL) {
 		gda_data_source_info_free (dsn_info);
-		gnome_db_show_error (_("An existing data source is already registered under the same name"));
+		gnome_db_show_error (NULL, _("An existing data source is already registered under the same name"));
 		gtk_widget_grab_focus (druid->priv->general_name);
 		return TRUE;
 	}
 
-	for (i = 0; i < strlen (name); i++) {
-		if (g_ascii_isspace (name[i])) {
-			gnome_db_show_error (_("Data source names cannot contain spaces"));
-			gtk_widget_grab_focus (druid->priv->general_name);
-			return TRUE;
-		} else if (!g_ascii_isalnum (name[i])) {
-			gnome_db_show_error (_("Data source names can only contain ASCII letters and numbers"));
-			gtk_widget_grab_focus (druid->priv->general_name);
-			return TRUE;
-		}
-	}
-
 	provider =  gnome_db_provider_selector_get_selected_provider (
 		GNOME_DB_PROVIDER_SELECTOR (druid->priv->general_provider));
 	if (!provider) {
-		gnome_db_show_error (_("You must select a provider for the new data source"));
+		gnome_db_show_error (NULL, _("You must select a provider for the new data source"));
 		gtk_widget_grab_focus (druid->priv->general_provider);
 		return TRUE;
 	}
@@ -390,59 +279,24 @@ general_next_pressed_cb (GnomeDruidPage *druid_page,
 	if (!prov_info) {
 		str = g_strdup_printf (_("Could not find provider %s in the current setup"),
 				       provider);
-		gnome_db_show_error (str);
+		gnome_db_show_error (NULL, str);
 		g_free (str);
 		gtk_widget_grab_focus (druid->priv->general_provider);
 		return TRUE;
 	}
 
-	g_hash_table_foreach_remove (druid->priv->provider_params,
-				     (GHRFunc) remove_hash_func,
-				     NULL);
-	if (GTK_IS_WIDGET (druid->priv->provider_detail))
-		gtk_widget_destroy (druid->priv->provider_detail);
-
-	druid->priv->provider_detail = gnome_db_new_vbox_widget (TRUE, 0);
-	for (l = prov_info->gda_params; l != NULL; l = l->next) {
-		GdaProviderParameterInfo *param_info;
-		GtkWidget *box;
-		GtkWidget *label;
-		GtkWidget *widget;
-
-		param_info = (GdaProviderParameterInfo *) l->data;
-		if (param_info) {
-			GHashTable *values;
-			gchar *title;
-
-			box = gnome_db_new_hbox_widget (TRUE, 0);
-			title = g_strdup_printf ("_%s:", param_info->short_description);
-			label = gnome_db_new_label_widget (title);
-			g_free (title);
-			gtk_box_pack_start (GTK_BOX (box), label, TRUE, TRUE, 0);
-			widget = gnome_db_new_entry_widget (0, TRUE);
-			gtk_label_set_mnemonic_widget (GTK_LABEL (label), widget);
-
-			/* set remembered value */
-			if ((values = g_hash_table_lookup (druid->priv->remembered_values, provider))) {
-				gtk_entry_set_text (GTK_ENTRY (widget),
-						    g_hash_table_lookup (values, param_info->name));
-			}
-
-			gtk_box_pack_start (GTK_BOX (box), widget, TRUE, TRUE, 0);
-			gtk_box_pack_start (GTK_BOX (druid->priv->provider_detail),
-					    box, TRUE, TRUE, 0);
-
-			g_hash_table_insert (druid->priv->provider_params,
-					     g_strdup (param_info->name),
-					     widget);
-		}
+	if (!druid->priv->provider_detail) {
+		druid->priv->provider_detail = gnome_db_dsn_spec_new (provider);
+		gtk_box_pack_start (GTK_BOX (druid->priv->provider_container),
+				    druid->priv->provider_detail, TRUE, TRUE, 0);
+		gtk_widget_show (druid->priv->provider_detail);
 	}
+	else
+		gnome_db_dsn_spec_set_provider (GNOME_DB_DSN_SPEC (druid->priv->provider_detail), provider);
 
-	gtk_box_pack_start (GTK_BOX (druid->priv->provider_container),
-			    druid->priv->provider_detail, TRUE, TRUE, 0);
 
-	str = get_specs_database_creation (druid);
-	if (str) 
+	op = get_specs_database_creation (druid);
+	if (op) 
 		return FALSE;
 	else {
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (druid->priv->choose_toggle), FALSE);
@@ -459,39 +313,18 @@ choose_next_pressed_cb (GnomeDruidPage *druid_page,
 	g_return_val_if_fail (GNOME_DB_IS_DSN_CONFIG_DRUID (druid), FALSE);
 
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (druid->priv->choose_toggle))) {
-		/* prepare a new data set for database creation parameters */
-		gchar *str;
-
 		if (! druid->priv->newdb_params) {
 			if (!druid->priv->client)
 				druid->priv->client = gda_client_new ();
+			GdaServerOperation *op;
 			
-			str = gda_client_get_specs_to_create_database (druid->priv->client, 
-								       gnome_db_provider_selector_get_selected_provider (
-									       GNOME_DB_PROVIDER_SELECTOR (druid->priv->general_provider)));
-			if (str) {
-				GnomeDbDataSet *dset;
-				GError *error = NULL;
-				
-				dset = GNOME_DB_DATA_SET (gnome_db_data_set_new_from_spec (NULL, str, &error));
-				if (dset) {
-					druid->priv->newdb_params = gnome_db_basic_form_new (NULL, dset);
-					gnome_db_basic_form_show_entries_actions (GNOME_DB_BASIC_FORM (druid->priv->newdb_params),
-										  FALSE);
-					g_object_unref (dset);
-				}
-				else {
-					gchar *str2;
-					str2 = g_strdup_printf ("Provider internal error: %s", 
-								error && error->message ? error->message : "???" );
-					druid->priv->newdb_params = gtk_label_new (str2);
-					g_free (str2);
-				}
-				
+			op = get_specs_database_creation (druid);
+			if (op) {
+				druid->priv->newdb_params = gnome_db_server_operation_new (op);
 				gtk_widget_show (druid->priv->newdb_params);
-				gtk_box_pack_start (GTK_BOX (druid->priv->newdb_box), druid->priv->newdb_params,
+				gtk_box_pack_start (GTK_BOX (druid->priv->newdb_box), 
+						    druid->priv->newdb_params,
 						    TRUE, TRUE, 0);
-				g_free (str);
 			}
 		}
 		gnome_druid_set_page (GNOME_DRUID (druid), GNOME_DRUID_PAGE (druid->priv->newdb_page));
@@ -514,51 +347,20 @@ newdb_next_pressed_cb (GnomeDruidPage *druid_page,
 	return TRUE;
 }
 
-static void
-add_remembered_hash (gpointer key, gpointer value, GnomeDbDsnConfigDruid *druid)
-{
-	GHashTable *htable;
-	const gchar *provider;
-	const gchar *param_name = key;
-	const gchar *entry_value;
-	GtkWidget *entry = (GtkWidget *) value;
-
-	provider = gnome_db_provider_selector_get_selected_provider (
-		GNOME_DB_PROVIDER_SELECTOR (druid->priv->general_provider));
-	entry_value = gtk_entry_get_text (GTK_ENTRY (entry));
-
-	htable = g_hash_table_lookup (druid->priv->remembered_values, provider);
-	if (htable) {
-		gpointer orig_key, orig_value;
-
-		if (g_hash_table_lookup_extended (htable, param_name, &orig_key, &orig_value)) {
-			g_hash_table_remove (htable, param_name);
-			g_free (orig_key);
-			g_free (orig_value);
-		}
-	} else {
-		htable = g_hash_table_new (g_str_hash, g_str_equal);
-		g_hash_table_insert (druid->priv->remembered_values, g_strdup (provider), htable);
-	}
-
-	g_hash_table_insert (htable, g_strdup (param_name), g_strdup (entry_value));
-}
-
 static gboolean
 provider_back_pressed_cb (GnomeDruidPage *druid_page,
 			  GtkWidget *druid_widget,
 			  GnomeDbDsnConfigDruid *druid)
 {
-	gchar *str;
+	GdaServerOperation *op;
 	g_return_val_if_fail (GNOME_DB_IS_DSN_CONFIG_DRUID (druid), TRUE);
 
-	g_hash_table_foreach (druid->priv->provider_params, (GHFunc) add_remembered_hash, druid);
 
 	/*
 	 * is the database creation supported by the chosen provider?
 	 */
-	str = get_specs_database_creation (druid);
-	if (str) {
+	op = get_specs_database_creation (druid);
+	if (op) {
 		/* database creation is supported */
 		gnome_druid_set_page (GNOME_DRUID (druid), GNOME_DRUID_PAGE (druid->priv->choose_page));
 	}
@@ -619,9 +421,7 @@ gnome_db_dsn_config_druid_init (GnomeDbDsnConfigDruid *druid,
 	druid->priv = g_new0 (GnomeDbDsnConfigDruidPrivate, 1);
 	druid->priv->dsn_info = g_new0 (GdaDataSourceInfo, 1);
 	druid->priv->provider_detail = NULL;
-	druid->priv->provider_params = g_hash_table_new (g_str_hash, g_str_equal);
-	druid->priv->remembered_values = g_hash_table_new (g_str_hash, g_str_equal);
-	druid->priv->create_db_spec = NULL;
+	druid->priv->create_db_op = NULL;
 
 	/* load icons */
 	druid->priv->logo = gdk_pixbuf_new_from_file (LIBGNOMEDB_ICONSDIR "/gnome-db.png", NULL);
@@ -832,20 +632,8 @@ gnome_db_dsn_config_druid_finalize (GObject *object)
 	if (druid->priv->dsn_info)
 		gda_data_source_info_free (druid->priv->dsn_info);
 
-	g_hash_table_foreach_remove (druid->priv->provider_params,
-				     (GHRFunc) remove_hash_func,
-				     NULL);
-	g_hash_table_destroy (druid->priv->provider_params);
-	druid->priv->provider_params = NULL;
-
-	g_hash_table_foreach_remove (druid->priv->remembered_values,
-				     (GHRFunc) remove_remembered_hash,
-				     NULL);
-	g_hash_table_destroy (druid->priv->remembered_values);
-	druid->priv->remembered_values = NULL;
-
-	if (druid->priv->create_db_spec)
-		g_free (druid->priv->create_db_spec);
+	if (druid->priv->create_db_op)
+		g_object_unref (druid->priv->create_db_op);
 
 	g_free (druid->priv);
 	druid->priv = NULL;
