@@ -24,11 +24,20 @@
 #include <libgda/libgda.h>
 #include <glib/gi18n-lib.h>
 #include "gnome-db-basic-form.h"
+#include "marshal.h"
 
 static void gnome_db_sql_console_class_init (GnomeDbSqlConsoleClass * class);
 static void gnome_db_sql_console_init (GnomeDbSqlConsole * wid);
 static void gnome_db_sql_console_dispose (GObject   * object);
 
+enum
+{
+        SQL_ENTERED,
+        LAST_SIGNAL
+};
+
+static gint gnome_db_sql_console_signals[LAST_SIGNAL] = { 0 };
+
 typedef struct {
 	gchar       *cmde;
 	gchar       *sql;
@@ -80,6 +89,38 @@ gnome_db_sql_console_get_type (void)
 	return type;
 }
 
+static gboolean
+sql_entered_accumulator (GSignalInvocationHint *ihint,
+			 GValue *return_accu,
+			 const GValue *handler_return,
+			 gpointer data)
+{
+	if (g_value_get_string (handler_return)) {
+		if (g_value_get_string (return_accu)) {
+			gchar *str;
+
+			str = g_strdup_printf ("%s%s\n", g_value_get_string (return_accu),
+					       g_value_get_string (handler_return));
+			g_value_take_string (return_accu, str);
+		}
+		else {
+			gchar *str;
+
+			str = g_strdup_printf ("%s\n", g_value_get_string (handler_return));
+			g_value_take_string (return_accu, str);
+		}
+	}
+
+	return TRUE; /* don't stop signal emission */
+}
+
+static gchar *
+m_sql_entered_cb (GnomeDbSqlConsole *console, const gchar *sql)
+{
+	/* do nothing */
+	return NULL;
+}
+
 static void
 gnome_db_sql_console_class_init (GnomeDbSqlConsoleClass * class)
 {
@@ -87,6 +128,18 @@ gnome_db_sql_console_class_init (GnomeDbSqlConsoleClass * class)
 	
 	parent_class = g_type_class_peek_parent (class);
 
+	/* signals */
+	gnome_db_sql_console_signals[SQL_ENTERED] =
+                g_signal_new ("sql_entered",
+                              G_TYPE_FROM_CLASS (object_class),
+                              G_SIGNAL_RUN_LAST,
+                              G_STRUCT_OFFSET (GnomeDbSqlConsoleClass, sql_entered),
+                              sql_entered_accumulator, NULL,
+                              marshal_STRING__STRING, G_TYPE_STRING, 1,
+                              G_TYPE_STRING);
+
+        class->sql_entered = m_sql_entered_cb;
+
 	object_class->dispose = gnome_db_sql_console_dispose;
 }
 
@@ -118,10 +171,10 @@ gnome_db_sql_console_new (GdaDict *dict, const gchar *message)
 	obj = g_object_new (GNOME_DB_TYPE_SQL_CONSOLE, NULL);
 	console = GNOME_DB_SQL_CONSOLE (obj);
 
-	console->priv->dict = ASSERT_DICT (dict);
-	
-	g_object_weak_ref (G_OBJECT (console->priv->dict),
-			   (GWeakNotify) gda_dict_weak_notify, console);
+	console->priv->dict = dict;
+	if (dict)
+		g_object_weak_ref (G_OBJECT (console->priv->dict),
+				   (GWeakNotify) gda_dict_weak_notify, console);
 
 	gnome_db_sql_console_initialize (console, message);
 
@@ -142,7 +195,7 @@ gnome_db_sql_console_dispose (GObject *object)
 	GnomeDbSqlConsole *console;
 
 	g_return_if_fail (object != NULL);
-	g_return_if_fail (IS_GNOME_DB_SQL_CONSOLE (object));
+	g_return_if_fail (GNOME_DB_IS_SQL_CONSOLE (object));
 	console = GNOME_DB_SQL_CONSOLE (object);
 
 	if (console->priv) {
@@ -439,6 +492,28 @@ run_sql_cmde (GnomeDbSqlConsole *console, HistItem *hist_item, const gchar *sql)
 	GdaParameterList *dset;
 	GError *error = NULL;
 	GdaDataModel *model;
+	gchar *sigret = NULL;
+
+#ifdef debug_signal
+	g_print (">> 'SQL_ENTERED' from %s\n", __FUNCTION__);
+#endif
+	g_signal_emit (G_OBJECT (console), gnome_db_sql_console_signals[SQL_ENTERED], 0, sql, &sigret);
+#ifdef debug_signal
+	g_print ("<< 'SQL_ENTERED' from %s\n", __FUNCTION__);
+#endif
+
+	/* output the return from signals with a '\n' at the end */
+	gtk_text_buffer_get_end_iter (console->priv->buffer, &iter);
+	gtk_text_buffer_insert_with_tags_by_name (console->priv->buffer, &iter, "\n", 1, "result-ok", NULL);
+	if (sigret) {
+		gtk_text_buffer_insert_with_tags_by_name (console->priv->buffer, &iter, sigret, -1, "result-ok", NULL);
+		g_free (sigret);
+	}
+
+	if (!console->priv->dict) {
+		gtk_text_buffer_insert_with_tags_by_name (console->priv->buffer, &iter, "\n", 1, "result-ok", NULL);
+		return;
+	}
 
 	cnc = gda_dict_get_connection (console->priv->dict);
 	if (!cnc) {
@@ -449,16 +524,15 @@ run_sql_cmde (GnomeDbSqlConsole *console, HistItem *hist_item, const gchar *sql)
 		g_warning ("Connection closed, can't run query");
 		return;
 	}
-
-	gtk_text_buffer_get_end_iter (console->priv->buffer, &iter);
 	
 	/* run query */
 	query = gda_query_new_from_sql (console->priv->dict, sql, NULL);
 	dset = gda_query_get_parameter_list (query);
 	if (fill_context_in_dialog (console, dset) != GTK_RESPONSE_ACCEPT) {
-		gtk_text_buffer_insert_with_tags_by_name (console->priv->buffer, &iter, "\n", 1, "result-error", NULL);
-		gtk_text_buffer_insert_with_tags_by_name (console->priv->buffer, &iter, _("Cancelled"), -1, "result-error", NULL);
-		gtk_text_buffer_insert_with_tags_by_name (console->priv->buffer, &iter, "\n\n", 2, "result-error", NULL);
+		gtk_text_buffer_insert_with_tags_by_name (console->priv->buffer, &iter, _("Cancelled"), -1, 
+							  "result-error", NULL);
+		gtk_text_buffer_insert_with_tags_by_name (console->priv->buffer, &iter, "\n\n", 2, 
+							  "result-error", NULL);
 	}
 	else {
 		gchar *rsql;
@@ -468,7 +542,7 @@ run_sql_cmde (GnomeDbSqlConsole *console, HistItem *hist_item, const gchar *sql)
 			GString *string;
 			GSList *list;
 
-			string = g_string_new ("\n");
+			string = g_string_new ("");
 			list = dset->parameters;
 			while (list) {
 				gchar *str;
@@ -791,7 +865,7 @@ gnome_db_sql_console_clear (GnomeDbSqlConsole *console)
 	GtkTextIter start, end;
 	GList *list;
 
-	g_return_if_fail (console && IS_GNOME_DB_SQL_CONSOLE (console));
+	g_return_if_fail (console && GNOME_DB_IS_SQL_CONSOLE (console));
 	g_return_if_fail (console->priv);
 	
 	mark = gtk_text_buffer_get_mark (console->priv->buffer, "sql_start");
@@ -824,7 +898,7 @@ gnome_db_sql_console_clear (GnomeDbSqlConsole *console)
 void
 gnome_db_sql_console_clear_history (GnomeDbSqlConsole *console)
 {
-	g_return_if_fail (console && IS_GNOME_DB_SQL_CONSOLE (console));
+	g_return_if_fail (console && GNOME_DB_IS_SQL_CONSOLE (console));
 	g_return_if_fail (console->priv);
 
 	if (console->priv->hist_items) {
