@@ -38,6 +38,21 @@ extern GHashTable *gnome_db_plugins_hash;
 
 static GHashTable *init_plugins_hash (void);
 
+/**
+ * gnome_db_util_init_plugins
+ *
+ * Forces the library to parse all the plugin files and build the %gnome_db_plugins_hash
+ * global hash table.
+ *
+ * Calling this function prior to gnome_db_util_new_data_entry () or gnome_db_util_new_cell_renderer() 
+ * is not necessary as it is called automatically.
+ */
+void
+gnome_db_util_init_plugins ()
+{
+	gnome_db_plugins_hash = init_plugins_hash ();
+}
+
 /**
  * gnome_db_util_new_data_entry
  * @dict: a #GdaDict object, or %NULL
@@ -322,8 +337,13 @@ init_plugins_hash (void)
 
 						plugin = (GnomeDbPlugin *)(list->data);
 						g_hash_table_insert (hash, plugin->plugin_name, plugin);
-						g_print ("  - loaded %s (%s)\n", plugin->plugin_name,
+						g_print ("  - loaded %s (%s):", plugin->plugin_name,
 							 plugin->plugin_descr);
+						if (plugin->entry_create_func)
+							g_print (" Entry");
+						if (plugin->cell_create_func)
+							g_print (" Cell");
+						g_print ("\n");
 						plugin->plugin_file = g_strdup (path);
 					}
 					
