@@ -123,7 +123,7 @@ gnome_db_canvas_join_dispose (GObject *object)
 {
 	GnomeDbCanvasJoin *cc;
 	g_return_if_fail (object != NULL);
-	g_return_if_fail (IS_GNOME_DB_CANVAS_JOIN (object));
+	g_return_if_fail (GNOME_DB_IS_CANVAS_JOIN (object));
 	
 	cc = GNOME_DB_CANVAS_JOIN (object);
 
@@ -146,7 +146,7 @@ gnome_db_canvas_join_finalize (GObject *object)
 {
 	GnomeDbCanvasJoin *cc;
 	g_return_if_fail (object != NULL);
-	g_return_if_fail (IS_GNOME_DB_CANVAS_JOIN (object));
+	g_return_if_fail (GNOME_DB_IS_CANVAS_JOIN (object));
 
 	cc = GNOME_DB_CANVAS_JOIN (object);
 	if (cc->priv) {
@@ -855,7 +855,7 @@ entity_item_moved_cb (GnomeCanvasItem *entity, GnomeDbCanvasJoin *cc)
 static void
 gnome_db_canvas_join_set_join (GnomeDbCanvasJoin *cc, GdaQueryJoin *join)
 {
-	g_return_if_fail (cc && IS_GNOME_DB_CANVAS_JOIN (cc));
+	g_return_if_fail (cc && GNOME_DB_IS_CANVAS_JOIN (cc));
 	g_return_if_fail (cc->priv);
 	if (join)
 		g_return_if_fail (GDA_IS_QUERY_JOIN (join ));
