@@ -116,7 +116,7 @@ gnome_db_canvas_query_struct_dispose (GObject *object)
 	GnomeDbCanvasQueryStruct *canvas;
 
 	g_return_if_fail (object != NULL);
-	g_return_if_fail (IS_GNOME_DB_CANVAS_QUERY_STRUCT (object));
+	g_return_if_fail (GNOME_DB_IS_CANVAS_QUERY_STRUCT (object));
 
 	canvas = GNOME_DB_CANVAS_QUERY_STRUCT (object);
 
@@ -135,7 +135,7 @@ gnome_db_canvas_query_struct_finalize (GObject *object)
 	GnomeDbCanvasQueryStruct *canvas;
 
 	g_return_if_fail (object != NULL);
-	g_return_if_fail (IS_GNOME_DB_CANVAS_QUERY_STRUCT (object));
+	g_return_if_fail (GNOME_DB_IS_CANVAS_QUERY_STRUCT (object));
 
 	canvas = GNOME_DB_CANVAS_QUERY_STRUCT (object);
 
@@ -164,9 +164,9 @@ drag_action_dcb (GnomeDbCanvas *canvas, GnomeDbCanvasItem *from_item, GnomeDbCan
 	GdaQueryField *qfield;
 	
 	/* fields retreival */
-	if (IS_GNOME_DB_CANVAS_FIELD (from_item))
+	if (GNOME_DB_IS_CANVAS_FIELD (from_item))
 		f_field = gnome_db_canvas_field_get_field (GNOME_DB_CANVAS_FIELD (from_item));
-	if (IS_GNOME_DB_CANVAS_FIELD (to_item))
+	if (GNOME_DB_IS_CANVAS_FIELD (to_item))
 		t_field = gnome_db_canvas_field_get_field (GNOME_DB_CANVAS_FIELD (to_item));
 	
 	if (!f_field || !t_field)
@@ -587,7 +587,7 @@ set_pixels_per_unit (GnomeDbCanvas *canvas, gdouble n)
 
 	list = GNOME_CANVAS_GROUP (GNOME_CANVAS (canvas)->root)->item_list;
 	while (list) {
-		if (IS_GNOME_DB_CANVAS_ENTITY (list->data))
+		if (GNOME_DB_IS_CANVAS_ENTITY (list->data))
 			g_object_set (G_OBJECT (list->data), "scale", n, NULL);
 		list = g_list_next (list);
 	}
