@@ -256,7 +256,7 @@ gnome_db_canvas_dispose (GObject   * object)
 	GnomeDbCanvas *canvas;
 
 	g_return_if_fail (object != NULL);
-	g_return_if_fail (IS_GNOME_DB_CANVAS (object));
+	g_return_if_fail (GNOME_DB_IS_CANVAS (object));
 
 	canvas = GNOME_DB_CANVAS (object);
 
@@ -278,7 +278,7 @@ gnome_db_canvas_finalize (GObject   * object)
 	GnomeDbCanvas *canvas;
 
 	g_return_if_fail (object != NULL);
-	g_return_if_fail (IS_GNOME_DB_CANVAS (object));
+	g_return_if_fail (GNOME_DB_IS_CANVAS (object));
 	canvas = GNOME_DB_CANVAS (object);
 
 	if (canvas->priv) {
@@ -412,7 +412,7 @@ gnome_db_canvas_post_init  (GnomeDbCanvas *canvas)
 GdaGraph *
 gnome_db_canvas_get_graph (GnomeDbCanvas *canvas)
 {
-	g_return_val_if_fail (canvas && IS_GNOME_DB_CANVAS (canvas), NULL);
+	g_return_val_if_fail (canvas && GNOME_DB_IS_CANVAS (canvas), NULL);
 	g_return_val_if_fail (canvas->priv, NULL);
 
 	return canvas->priv->graph;
@@ -433,9 +433,9 @@ gnome_db_canvas_get_graph (GnomeDbCanvas *canvas)
 void
 gnome_db_canvas_declare_item (GnomeDbCanvas *canvas, GnomeDbCanvasItem *item)
 {
-	g_return_if_fail (canvas && IS_GNOME_DB_CANVAS (canvas));
+	g_return_if_fail (canvas && GNOME_DB_IS_CANVAS (canvas));
 	g_return_if_fail (canvas->priv);
-	g_return_if_fail (item && IS_GNOME_DB_CANVAS_ITEM (item));
+	g_return_if_fail (item && GNOME_DB_IS_CANVAS_ITEM (item));
 
 	if (g_slist_find (canvas->priv->items, item))
 		return;
@@ -493,7 +493,7 @@ GtkWidget *
 gnome_db_canvas_set_in_scrolled_window (GnomeDbCanvas *canvas)
 {
 	GtkWidget *sw;
-	g_return_val_if_fail (canvas && IS_GNOME_DB_CANVAS (canvas), NULL);
+	g_return_val_if_fail (canvas && GNOME_DB_IS_CANVAS (canvas), NULL);
 
 	sw = gtk_scrolled_window_new (NULL, NULL);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw), 
@@ -636,7 +636,7 @@ void
 gnome_db_canvas_set_zoom_factor (GnomeDbCanvas *canvas, gdouble n)
 {
 	GnomeDbCanvasClass *class = GNOME_DB_CANVAS_CLASS (G_OBJECT_GET_CLASS (canvas));
-	g_return_if_fail (canvas && IS_GNOME_DB_CANVAS (canvas));
+	g_return_if_fail (canvas && GNOME_DB_IS_CANVAS (canvas));
 
 	gnome_canvas_set_pixels_per_unit (GNOME_CANVAS (canvas), n);
 	if (class->set_pixels_per_unit)
@@ -654,7 +654,7 @@ gnome_db_canvas_set_zoom_factor (GnomeDbCanvas *canvas, gdouble n)
 gdouble
 gnome_db_canvas_get_zoom_factor (GnomeDbCanvas *canvas)
 {
-	g_return_val_if_fail (canvas && IS_GNOME_DB_CANVAS (canvas), 1.);
+	g_return_val_if_fail (canvas && GNOME_DB_IS_CANVAS (canvas), 1.);
 
 	return GNOME_CANVAS (canvas)->pixels_per_unit;
 }
@@ -674,7 +674,7 @@ gnome_db_canvas_fit_zoom_factor (GnomeDbCanvas *canvas)
 	gdouble zoom, xall, yall;
 	gdouble wx1, wy1, wx2, wy2;
 
-	g_return_val_if_fail (canvas && IS_GNOME_DB_CANVAS (canvas), 1.);	
+	g_return_val_if_fail (canvas && GNOME_DB_IS_CANVAS (canvas), 1.);	
 
 	xall = GTK_WIDGET (canvas)->allocation.width;
 	yall = GTK_WIDGET (canvas)->allocation.height;
